@@ -27,26 +27,6 @@
 #include "dvb-v5-std.h"
 #include "dvb-scan.h"
 
-static const char *parm_name(const struct parse_table *table)
-{
-	if (table->prop < DTV_MAX_COMMAND)
-		return dvb_v5_name[table->prop];
-	switch (table->prop) {
-	case DTV_CH_NAME:
-		return ("CHANNEL");
-	case DTV_POLARIZATION:
-		return ("POLARIZATION");
-	case DTV_VIDEO_PID:
-		return ("VIDEO PID");
-	case DTV_AUDIO_PID:
-		return ("AUDIO PID");
-	case DTV_SERVICE_ID:
-		return ("SERVICE ID");
-	default:
-		return ("unknown");
-	}
-}
-
 /*
  * Generic parse function for all formats each channel is contained into
  * just one line.
@@ -78,6 +58,7 @@ struct dvb_file *parse_format_oneline(const char *fname,
 	fd = fopen(fname, "r");
 	if (!fd) {
 		perror(fname);
+		free(dvb_file);
 		return NULL;
 	}
 
@@ -145,7 +126,7 @@ struct dvb_file *parse_format_oneline(const char *fname,
 				p = strtok(NULL, delimiter);
 			if (!p) {
 				sprintf(err_msg, "parameter %i (%s) missing",
-					i, parm_name(table));
+					i, dvb_cmd_name(table->prop));
 				goto error;
 			}
 			if (table->size) {
@@ -154,7 +135,7 @@ struct dvb_file *parse_format_oneline(const char *fname,
 						break;
 				if (j == table->size) {
 					sprintf(err_msg, "parameter %s invalid: %s",
-						parm_name(table), p);
+						dvb_cmd_name(table->prop), p);
 					goto error;
 				}
 				if (table->prop == DTV_BANDWIDTH_HZ)
@@ -377,6 +358,29 @@ error:
 	return -1;
 }
 
+static int store_entry_prop(struct dvb_entry *entry,
+			    uint32_t cmd, uint32_t value)
+{
+	int i;
+
+	for (i = 0; i < entry->n_props; i++) {
+		if (cmd == entry->props[i].cmd)
+			break;
+	}
+	if (i == entry->n_props) {
+		if (i == DTV_MAX_COMMAND) {
+			fprintf(stderr, "Can't add property %s\n",
+			       dvb_v5_name[cmd]);
+			return -1;
+		}
+		entry->n_props++;
+		entry->props[i].cmd = cmd;
+	}
+
+	entry->props[i].u.data = value;
+
+	return 0;
+}
 
 #define CHANNEL "CHANNEL"
 
@@ -412,31 +416,6 @@ static int fill_entry(struct dvb_entry *entry, char *key, char *value)
 		return 0;
 	}
 
-	/* Handle the DVB extra DTV_foo properties */
-	for (i = 0; i < ARRAY_SIZE(dvb_user_name); i++) {
-		if (!dvb_user_name[i])
-			continue;
-		if (!strcasecmp(key, dvb_user_name[i]))
-			break;
-	}
-	if (i < ARRAY_SIZE(dvb_user_name)) {
-		const char * const *attr_name = dvb_attr_names(i);
-		n_prop = entry->n_props;
-		entry->props[n_prop].cmd = i + DTV_USER_COMMAND_START;
-		if (!attr_name || !*attr_name)
-			entry->props[n_prop].u.data = atol(value);
-		else {
-			for (j = 0; attr_name[j]; j++)
-				if (!strcasecmp(value, attr_name[j]))
-					break;
-			if (!attr_name[j])
-				return -2;
-			entry->props[n_prop].u.data = j + DTV_USER_COMMAND_START;
-		}
-		entry->n_props++;
-		return 0;
-	}
-
 	/* Handle the other properties */
 
 	if (!strcasecmp(key, "SERVICE_ID")) {
@@ -473,16 +452,15 @@ static int fill_entry(struct dvb_entry *entry, char *key, char *value)
 		is_video = 1;
 	else if (!strcasecmp(key, "AUDIO_PID"))
 		is_audio = 1;
-	/*else if (!strcasecmp(key, "POLARIZATION")) {
-		entry->service_id = atol(value);
-		for (j = 0; ARRAY_SIZE(pol_name); j++)
-			if (!strcasecmp(value, pol_name[j]))
+	else if (!strcasecmp(key, "POLARIZATION")) {
+		for (j = 0; ARRAY_SIZE(dvb_sat_pol_name); j++)
+			if (!strcasecmp(value, dvb_sat_pol_name[j]))
 				break;
-		if (j == ARRAY_SIZE(pol_name))
+		if (j == ARRAY_SIZE(dvb_sat_pol_name))
 			return -2;
-		entry->pol = j;
+		store_entry_prop(entry, DTV_POLARIZATION, j);
 		return 0;
-	}*/ else if (!strncasecmp(key,"PID_", 4)){
+	} else if (!strncasecmp(key,"PID_", 4)){
 		type = strtol(&key[4], NULL, 16);
 		if (!type)
 			return 0;
@@ -558,6 +536,7 @@ struct dvb_file *read_dvb_file(const char *fname)
 	fd = fopen(fname, "r");
 	if (!fd) {
 		perror(fname);
+		free(dvb_file);
 		return NULL;
 	}
 
@@ -623,9 +602,6 @@ struct dvb_file *read_dvb_file(const char *fname)
 				sprintf(err_msg, "value %s is invalid for %s",
 					value, key);
 				goto error;
-			} else if (rc == -2) {
-				sprintf(err_msg, "key %s is unknown", key);
-				goto error;
 			}
 		}
 	} while (1);
@@ -692,11 +668,6 @@ int write_dvb_file(const char *fname, struct dvb_file *dvb_file)
 			fprintf(fp, "\n");
 		}
 
-		/*if (entry->pol != POLARIZATION_OFF) {*/
-			/*fprintf(fp, "\tPOLARIZATION = %s\n",*/
-				/*pol_name[entry->pol]);*/
-		/*}*/
-
 		if (entry->sat_number >= 0) {
 			fprintf(fp, "\tSAT_NUMBER = %d\n",
 				entry->sat_number);
@@ -796,29 +767,6 @@ char *dvb_vchannel(struct dvb_v5_descriptors *dvb_desc,
 	return buf;
 }
 
-static int store_entry_prop(struct dvb_entry *entry,
-			    uint32_t cmd, uint32_t value)
-{
-	int i;
-
-	for (i = 0; i < entry->n_props; i++) {
-		if (cmd == entry->props[i].cmd)
-			break;
-	}
-	if (i == entry->n_props) {
-		if (i == DTV_MAX_COMMAND) {
-			fprintf(stderr, "Can't add property %s\n",
-			       dvb_v5_name[cmd]);
-			return -1;
-		}
-		entry->n_props++;
-	}
-
-	entry->props[i].u.data = value;
-
-	return 0;
-}
-
 static void handle_std_specific_parms(struct dvb_entry *entry,
 				      struct dvb_v5_descriptors *dvb_desc)
 {
@@ -857,7 +805,6 @@ static void handle_std_specific_parms(struct dvb_entry *entry,
 				 nit_table->frequency[0]);
 		store_entry_prop(entry, DTV_MODULATION,
 				 nit_table->modulation);
-		/*entry->pol = nit_table->pol;*/
 		store_entry_prop(entry, DTV_POLARIZATION,
 				 nit_table->pol);
 		store_entry_prop(entry, DTV_DELIVERY_SYSTEM,
