@@ -20,8 +20,12 @@
 # Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
 */
 
+/* prevent GCC 4.7 inlining error */
+#undef _FORTIFY_SOURCE
+
 #define _LARGEFILE64_SOURCE 1
 
+#include <config.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <fcntl.h>
@@ -37,7 +41,7 @@
 #error open/read/mmap is a prepocessor macro !!
 #endif
 
-#if __GNUC__ >= 4
+#if HAVE_VISIBILITY
 #define LIBV4L_PUBLIC __attribute__ ((visibility("default")))
 #else
 #define LIBV4L_PUBLIC
@@ -46,12 +50,11 @@
 LIBV4L_PUBLIC int open(const char *file, int oflag, ...)
 {
 	int fd;
-	struct v4l2_capability cap;
 	int v4l_device = 0;
 
 	/* check if we're opening a video4linux2 device */
 	if (!strncmp(file, "/dev/video", 10) || !strncmp(file, "/dev/v4l/", 9)) {
-		/* Some apps open the device read only, but we need rw rights as the
+		/* Some apps open the device read-only, but we need rw rights as the
 		   buffers *MUST* be mapped rw */
 		oflag = (oflag & ~O_ACCMODE) | O_RDWR;
 		v4l_device = 1;
@@ -63,7 +66,7 @@ LIBV4L_PUBLIC int open(const char *file, int oflag, ...)
 		mode_t mode;
 
 		va_start(ap, oflag);
-		mode = va_arg(ap, mode_t);
+		mode = va_arg(ap, PROMOTED_MODE_T);
 
 		fd = SYS_OPEN(file, oflag, mode);
 
@@ -76,14 +79,6 @@ LIBV4L_PUBLIC int open(const char *file, int oflag, ...)
 	if (fd == -1 || !v4l_device)
 		return fd;
 
-	/* check that this is an v4l2 device, libv4l2 only supports v4l2 devices */
-	if (SYS_IOCTL(fd, VIDIOC_QUERYCAP, &cap))
-		return fd;
-
-	/* libv4l2 only adds functionality to capture capable devices */
-	if (!(cap.capabilities & V4L2_CAP_VIDEO_CAPTURE))
-		return fd;
-
 	/* Try to Register with libv4l2 (in case of failure pass the fd to the
 	   application as is) */
 	v4l2_fd_open(fd, 0);
@@ -102,7 +97,7 @@ LIBV4L_PUBLIC int open64(const char *file, int oflag, ...)
 		mode_t mode;
 
 		va_start(ap, oflag);
-		mode = va_arg(ap, mode_t);
+		mode = va_arg(ap, PROMOTED_MODE_T);
 
 		fd = open(file, oflag | O_LARGEFILE, mode);
 
