@@ -72,6 +72,9 @@ static const struct v4lcontrol_flags_info v4lcontrol_flags[] = {
 	{ 0x046d, 0x09b2, 0, "FUJITSU", "FJNB1C9",
 		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED, 0,
 		"FUJITSU", "LifeBook P7230" },
+	/* A re-branded ASUS notebook */
+	{ 0x04f2, 0xb012, 0, "Founder PC", "T14MF",
+		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED },
 	/* Note no whitespace padding for board vendor, this is not a typo */
 	{ 0x04f2, 0xb012, 0, "PEGATRON CORPORATION", "X71TL     ",
 		V4LCONTROL_HFLIPPED | V4LCONTROL_VFLIPPED },
@@ -268,7 +271,7 @@ static const char *asus_board_vendor[] = {
 static const char *asus_board_name[] = {
 	"A3[A-Z]*",
 	"B50[A-Z]*",
-	"F[3579][A-Z]*", "F70[A-Z]*", "F[58]2[A-Z]*",
+	"F[23579][A-Z]*", "F70[A-Z]*", "F[58]2[A-Z]*",
 	"G[12][A-Z]*", "G[57]0[A-Z]*",
 	"K[4567]0[A-Z]*", "K[56]1[A-Z]*", "K52[A-Z]*", "K[45]3[A-Z]*",
 	"N[12579]0[A-Z]*", "N[56]1[A-Z]*", "N82[A-Z]*", "N[47]3[A-Z]*",
@@ -314,6 +317,7 @@ static const struct v4lcontrol_usb_id asus_camera_id[] = {
 	{ 0x174f, 0x8a31 },
 	{ 0x174f, 0xa311 },
 	{ 0x1d4d, 0x1002 },
+	{ 0x05e1, 0x0501 },
 	{ 0x0000, 0x0000 }
 };
 
@@ -359,6 +363,13 @@ static int v4lcontrol_get_usb_info(struct v4lcontrol_data *data,
 	char sysfs_name[512];
 	char c, *s, buf[32];
 
+	snprintf(sysfs_name, sizeof(sysfs_name),
+	    "%s/sys/class/video4linux", sysfs_prefix);
+
+	/* Check for sysfs mounted before trying to search */
+	if (stat(sysfs_name, &st) != 0)
+		return 0; /* Not found, sysfs not mounted? */
+
 	if (fstat(data->fd, &st) || !S_ISCHR(st.st_mode))
 		return 0; /* Should never happen */
 
