@@ -15,6 +15,7 @@
  * Free Software Foundation; version 2 of the License.
  */
 
+#include <limits.h>
 #include <string.h>
 #include <unistd.h>
 #include "helper-funcs.h"
@@ -791,14 +792,13 @@ DCT_8x4(int *coeff, unsigned char *out)
 {
 	register int base, val1, val2, val3;
 	int tmp1, tmp2;
-	int C0, C4, C16, C20;
+	int C4, C16, C20;
 	int C2_18, C6_22, C1_17, C3_19, C5_21, C7_23;
 	register int t;
 #ifdef APPROXIMATE_MUL_BY_SHIFT
 	register int x;
 #endif
 
-	C0 = coeff[0];
 	C4 = coeff[4];
 	C16 = coeff[16];
 	C20 = coeff[20];
@@ -1455,7 +1455,11 @@ int main(int argc, char *argv[])
 
 
 		dest_size = width * height * 3 / 2;
-		if (dest_size > sizeof(dest_buf)) {
+		if (width <= 0 || width > SHRT_MAX || height <= 0 || height > SHRT_MAX) {
+			fprintf(stderr, "%s: error: width or height out of bounds\n",
+					argv[0]);
+			dest_size = -1;
+		} else if (dest_size > sizeof(dest_buf)) {
 			fprintf(stderr, "%s: error: dest_buf too small, need: %d\n",
 					argv[0], dest_size);
 			dest_size = -1;
