@@ -35,7 +35,7 @@
 	} while (0)
 
 void v4lconvert_rgb24_to_yuv420(const unsigned char *src, unsigned char *dest,
-		const struct v4l2_format *src_fmt, int bgr, int yvu)
+		const struct v4l2_format *src_fmt, int bgr, int yvu, int bpp)
 {
 	int x, y;
 	unsigned char *udest, *vdest;
@@ -47,9 +47,10 @@ void v4lconvert_rgb24_to_yuv420(const unsigned char *src, unsigned char *dest,
 				RGB2Y(src[2], src[1], src[0], *dest++);
 			else
 				RGB2Y(src[0], src[1], src[2], *dest++);
-			src += 3;
+			src += bpp;
 		}
-		src += src_fmt->fmt.pix.bytesperline - 3 * src_fmt->fmt.pix.width;
+
+		src += src_fmt->fmt.pix.bytesperline - bpp * src_fmt->fmt.pix.width;
 	}
 	src -= src_fmt->fmt.pix.height * src_fmt->fmt.pix.bytesperline;
 
@@ -66,19 +67,19 @@ void v4lconvert_rgb24_to_yuv420(const unsigned char *src, unsigned char *dest,
 		for (x = 0; x < src_fmt->fmt.pix.width / 2; x++) {
 			int avg_src[3];
 
-			avg_src[0] = (src[0] + src[3] + src[src_fmt->fmt.pix.bytesperline] +
-					src[src_fmt->fmt.pix.bytesperline + 3]) / 4;
-			avg_src[1] = (src[1] + src[4] + src[src_fmt->fmt.pix.bytesperline + 1] +
-					src[src_fmt->fmt.pix.bytesperline + 4]) / 4;
-			avg_src[2] = (src[2] + src[5] + src[src_fmt->fmt.pix.bytesperline + 2] +
-					src[src_fmt->fmt.pix.bytesperline + 5]) / 4;
+			avg_src[0] = (src[0] + src[bpp] + src[src_fmt->fmt.pix.bytesperline] +
+					src[src_fmt->fmt.pix.bytesperline + bpp]) / 4;
+			avg_src[1] = (src[1] + src[bpp + 1] + src[src_fmt->fmt.pix.bytesperline + 1] +
+					src[src_fmt->fmt.pix.bytesperline + bpp + 1]) / 4;
+			avg_src[2] = (src[2] + src[bpp + 2] + src[src_fmt->fmt.pix.bytesperline + 2] +
+					src[src_fmt->fmt.pix.bytesperline + bpp + 2]) / 4;
 			if (bgr)
 				RGB2UV(avg_src[2], avg_src[1], avg_src[0], *udest++, *vdest++);
 			else
 				RGB2UV(avg_src[0], avg_src[1], avg_src[2], *udest++, *vdest++);
-			src += 6;
+			src += 2 * bpp;
 		}
-		src += 2 * src_fmt->fmt.pix.bytesperline - 3 * src_fmt->fmt.pix.width;
+		src += 2 * src_fmt->fmt.pix.bytesperline - bpp * src_fmt->fmt.pix.width;
 	}
 }
 
@@ -200,12 +201,12 @@ void v4lconvert_yuv420_to_rgb24(const unsigned char *src, unsigned char *dest,
 }
 
 void v4lconvert_yuyv_to_bgr24(const unsigned char *src, unsigned char *dest,
-		int width, int height)
+		int width, int height, int stride)
 {
 	int j;
 
 	while (--height >= 0) {
-		for (j = 0; j < width; j += 2) {
+		for (j = 0; j + 1 < width; j += 2) {
 			int u = src[1];
 			int v = src[3];
 			int u1 = (((u - 128) << 7) +  (u - 128)) >> 6;
@@ -222,16 +223,17 @@ void v4lconvert_yuyv_to_bgr24(const unsigned char *src, unsigned char *dest,
 			*dest++ = CLIP(src[2] + v1);
 			src += 4;
 		}
+		src += stride - width * 2;
 	}
 }
 
 void v4lconvert_yuyv_to_rgb24(const unsigned char *src, unsigned char *dest,
-		int width, int height)
+		int width, int height, int stride)
 {
 	int j;
 
 	while (--height >= 0) {
-		for (j = 0; j < width; j += 2) {
+		for (j = 0; j + 1 < width; j += 2) {
 			int u = src[1];
 			int v = src[3];
 			int u1 = (((u - 128) << 7) +  (u - 128)) >> 6;
@@ -248,11 +250,12 @@ void v4lconvert_yuyv_to_rgb24(const unsigned char *src, unsigned char *dest,
 			*dest++ = CLIP(src[2] + u1);
 			src += 4;
 		}
+		src += stride - (width * 2);
 	}
 }
 
 void v4lconvert_yuyv_to_yuv420(const unsigned char *src, unsigned char *dest,
-		int width, int height, int yvu)
+		int width, int height, int stride, int yvu)
 {
 	int i, j;
 	const unsigned char *src1;
@@ -261,16 +264,17 @@ void v4lconvert_yuyv_to_yuv420(const unsigned char *src, unsigned char *dest,
 	/* copy the Y values */
 	src1 = src;
 	for (i = 0; i < height; i++) {
-		for (j = 0; j < width; j += 2) {
+		for (j = 0; j + 1 < width; j += 2) {
 			*dest++ = src1[0];
 			*dest++ = src1[2];
 			src1 += 4;
 		}
+		src1 += stride - width * 2;
 	}
 
 	/* copy the U and V values */
 	src++;				/* point to V */
-	src1 = src + width * 2;		/* next line */
+	src1 = src + stride;		/* next line */
 	if (yvu) {
 		vdest = dest;
 		udest = dest + width * height / 4;
@@ -279,24 +283,25 @@ void v4lconvert_yuyv_to_yuv420(const unsigned char *src, unsigned char *dest,
 		vdest = dest + width * height / 4;
 	}
 	for (i = 0; i < height; i += 2) {
-		for (j = 0; j < width; j += 2) {
+		for (j = 0; j + 1 < width; j += 2) {
 			*udest++ = ((int) src[0] + src1[0]) / 2;	/* U */
 			*vdest++ = ((int) src[2] + src1[2]) / 2;	/* V */
 			src += 4;
 			src1 += 4;
 		}
+		src1 += stride - width * 2;
 		src = src1;
-		src1 += width * 2;
+		src1 += stride;
 	}
 }
 
 void v4lconvert_yvyu_to_bgr24(const unsigned char *src, unsigned char *dest,
-		int width, int height)
+		int width, int height, int stride)
 {
 	int j;
 
 	while (--height >= 0) {
-		for (j = 0; j < width; j += 2) {
+		for (j = 0; j + 1 < width; j += 2) {
 			int u = src[3];
 			int v = src[1];
 			int u1 = (((u - 128) << 7) +  (u - 128)) >> 6;
@@ -313,16 +318,17 @@ void v4lconvert_yvyu_to_bgr24(const unsigned char *src, unsigned char *dest,
 			*dest++ = CLIP(src[2] + v1);
 			src += 4;
 		}
+		src += stride - (width * 2);
 	}
 }
 
 void v4lconvert_yvyu_to_rgb24(const unsigned char *src, unsigned char *dest,
-		int width, int height)
+		int width, int height, int stride)
 {
 	int j;
 
 	while (--height >= 0) {
-		for (j = 0; j < width; j += 2) {
+		for (j = 0; j + 1 < width; j += 2) {
 			int u = src[3];
 			int v = src[1];
 			int u1 = (((u - 128) << 7) +  (u - 128)) >> 6;
@@ -339,16 +345,17 @@ void v4lconvert_yvyu_to_rgb24(const unsigned char *src, unsigned char *dest,
 			*dest++ = CLIP(src[2] + u1);
 			src += 4;
 		}
+		src += stride - (width * 2);
 	}
 }
 
 void v4lconvert_uyvy_to_bgr24(const unsigned char *src, unsigned char *dest,
-		int width, int height)
+		int width, int height, int stride)
 {
 	int j;
 
 	while (--height >= 0) {
-		for (j = 0; j < width; j += 2) {
+		for (j = 0; j + 1 < width; j += 2) {
 			int u = src[0];
 			int v = src[2];
 			int u1 = (((u - 128) << 7) +  (u - 128)) >> 6;
@@ -365,16 +372,17 @@ void v4lconvert_uyvy_to_bgr24(const unsigned char *src, unsigned char *dest,
 			*dest++ = CLIP(src[3] + v1);
 			src += 4;
 		}
+		src += stride - width * 2;
 	}
 }
 
 void v4lconvert_uyvy_to_rgb24(const unsigned char *src, unsigned char *dest,
-		int width, int height)
+		int width, int height, int stride)
 {
 	int j;
 
 	while (--height >= 0) {
-		for (j = 0; j < width; j += 2) {
+		for (j = 0; j + 1 < width; j += 2) {
 			int u = src[0];
 			int v = src[2];
 			int u1 = (((u - 128) << 7) +  (u - 128)) >> 6;
@@ -391,11 +399,12 @@ void v4lconvert_uyvy_to_rgb24(const unsigned char *src, unsigned char *dest,
 			*dest++ = CLIP(src[3] + u1);
 			src += 4;
 		}
+		src += stride - width * 2;
 	}
 }
 
 void v4lconvert_uyvy_to_yuv420(const unsigned char *src, unsigned char *dest,
-		int width, int height, int yvu)
+		int width, int height, int stride, int yvu)
 {
 	int i, j;
 	const unsigned char *src1;
@@ -404,15 +413,16 @@ void v4lconvert_uyvy_to_yuv420(const unsigned char *src, unsigned char *dest,
 	/* copy the Y values */
 	src1 = src;
 	for (i = 0; i < height; i++) {
-		for (j = 0; j < width; j += 2) {
+		for (j = 0; j + 1 < width; j += 2) {
 			*dest++ = src1[1];
 			*dest++ = src1[3];
 			src1 += 4;
 		}
+		src1 += stride - width * 2;
 	}
 
 	/* copy the U and V values */
-	src1 = src + width * 2;		/* next line */
+	src1 = src + stride;		/* next line */
 	if (yvu) {
 		vdest = dest;
 		udest = dest + width * height / 4;
@@ -421,14 +431,15 @@ void v4lconvert_uyvy_to_yuv420(const unsigned char *src, unsigned char *dest,
 		vdest = dest + width * height / 4;
 	}
 	for (i = 0; i < height; i += 2) {
-		for (j = 0; j < width; j += 2) {
+		for (j = 0; j + 1 < width; j += 2) {
 			*udest++ = ((int) src[0] + src1[0]) / 2;	/* U */
 			*vdest++ = ((int) src[2] + src1[2]) / 2;	/* V */
 			src += 4;
 			src1 += 4;
 		}
+		src1 += stride - width * 2;
 		src = src1;
-		src1 += width * 2;
+		src1 += stride;
 	}
 }
 
@@ -576,6 +587,36 @@ void v4lconvert_rgb565_to_yuv420(const unsigned char *src, unsigned char *dest,
 	}
 }
 
+void v4lconvert_y16_to_rgb24(const unsigned char *src, unsigned char *dest,
+		int width, int height)
+{
+	int j;
+	while (--height >= 0) {
+		for (j = 0; j < width; j++) {
+			*dest++ = *src;
+			*dest++ = *src;
+			*dest++ = *src;
+			src+=2;
+		}
+	}
+}
+
+void v4lconvert_y16_to_yuv420(const unsigned char *src, unsigned char *dest,
+		const struct v4l2_format *src_fmt)
+{
+	int x, y;
+
+	/* Y */
+	for (y = 0; y < src_fmt->fmt.pix.height; y++)
+		for (x = 0; x < src_fmt->fmt.pix.width; x++){
+			*dest++ = *src;
+			src+=2;
+		}
+
+	/* Clear U/V */
+	memset(dest, 0x80, src_fmt->fmt.pix.width * src_fmt->fmt.pix.height / 2);
+}
+
 void v4lconvert_grey_to_rgb24(const unsigned char *src, unsigned char *dest,
 		int width, int height)
 {
@@ -605,7 +646,7 @@ void v4lconvert_grey_to_yuv420(const unsigned char *src, unsigned char *dest,
 }
 
 /* Unpack buffer of (vw bit) data into padded 16bit buffer. */
-static inline void convert_packed_to_16bit(uint8_t *raw, uint16_t *unpacked,
+static inline void convert_packed_to_16bit(const uint8_t *raw, uint16_t *unpacked,
 					   int vw, int unpacked_len)
 {
 	int mask = (1 << vw) - 1;
@@ -685,3 +726,25 @@ int v4lconvert_y10b_to_yuv420(struct v4lconvert_data *data,
 
 	return 0;
 }
+
+void v4lconvert_rgb32_to_rgb24(const unsigned char *src, unsigned char *dest,
+		int width, int height,int bgr)
+{
+	int j;
+	while (--height >= 0) {
+		for (j = 0; j < width; j++) {
+			if (bgr){
+				*dest++ = src[2];
+				*dest++ = src[1];
+				*dest++ = src[0];
+				src+=4;
+			}
+			else{
+				*dest++ = *src++;
+				*dest++ = *src++;
+				*dest++ = *src++;
+				src+=1;
+			}
+		}
+	}
+}
