@@ -51,6 +51,9 @@ struct arguments {
 	unsigned diseqc_wait, dont_add_new_freqs, timeout_multiply;
 	unsigned other_nit;
 	enum file_formats input_format, output_format;
+
+	/* Used by status print */
+	unsigned n_status_lines;
 };
 
 static const struct argp_option options[] = {
@@ -91,85 +94,82 @@ static int verbose = 0;
 		fprintf(stderr, " (%s)\n", strerror(errno));		\
 	} while (0)
 
-static char *sig_bits[7] = {
-	[0] = "RF sig",
-	[1] = "Carrier",
-	[2] = "Viterbi",
-	[3] = "Sync",
-	[4] = "Lock",
-	[5] = "Timeout",
-	[6] = "Reinit",
-};
-
-static void print_frontend_stats(struct dvb_v5_fe_parms *parms)
+static int print_frontend_stats(struct arguments *args,
+				struct dvb_v5_fe_parms *parms)
 {
-	char buf[256], *p = buf;
-	int i, s, len = sizeof(buf);
-	fe_status_t status;
-	uint32_t snr = 0, _signal = 0;
-	uint32_t ber = 0, ucb = 0;
+	char buf[512], *p;
+	int rc, i, len, show;
+	uint32_t status = 0;
 
-	if (dvb_fe_retrieve_stats(parms, DTV_STATUS, &status)) {
-		fprintf(stderr, "Error: dvb_fe_get_stats failed: no status\n");
-		return;
+	/* Move cursor up and cleans down */
+	if (isatty(STDERR_FILENO) && args->n_status_lines)
+		fprintf(stderr, "\r\x1b[%dA\x1b[J", args->n_status_lines);
+
+	args->n_status_lines = 0;
+
+	if (isatty(STDERR_FILENO)) {
+		rc = dvb_fe_retrieve_stats(parms, DTV_STATUS, &status);
+		if (rc)
+			status = 0;
+		if (status & FE_HAS_LOCK)
+			fprintf(stderr, "\x1b[1;32m");
+		else
+			fprintf(stderr, "\x1b[33m");
 	}
 
-	/* Get the most significative status */
-	for (i = ARRAY_SIZE(sig_bits) - 1; i >= 0 ; i--) {
-		if ((1 << i) & status) {
-			s = snprintf(p, len, "%-7s ", sig_bits[i]);
-			break;
+	p = buf;
+	len = sizeof(buf);
+	dvb_fe_snprintf_stat(parms,  DTV_STATUS, NULL, 0, &p, &len, &show);
+
+	for (i = 0; i < MAX_DTV_STATS; i++) {
+		show = 1;
+
+		dvb_fe_snprintf_stat(parms, DTV_QUALITY, "Quality",
+				     i, &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_STAT_SIGNAL_STRENGTH, "Signal",
+				     i, &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_STAT_CNR, "C/N",
+				     i, &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_STAT_ERROR_BLOCK_COUNT, "UCB",
+				     i,  &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_BER, "postBER",
+				     i,  &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_PRE_BER, "preBER",
+				     i,  &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_PER, "PER",
+				     i,  &p, &len, &show);
+
+		if (p != buf) {
+			if (args->n_status_lines)
+				fprintf(stderr, "\t%s\n", buf);
+			else
+				fprintf(stderr, "%s\n", buf);
+
+			args->n_status_lines++;
+
+			p = buf;
+			len = sizeof(buf);
 		}
 	}
-	if (i < 0)
-		s = snprintf(p, len, "%7s ", "");
-	p += s;
-	len -= s;
 
-	/* Add the status bits */
-	s = snprintf(p, len, "(0x%02x) ", status);
-	p += s;
-	len -= s;
+	fflush(stderr);
 
-	if (!dvb_fe_retrieve_stats(parms, DTV_SIGNAL_STRENGTH, &_signal)) {
-		s = snprintf(p, len, "| signal %03.2f%% ",
-			     (_signal * 100.0) / 0xffff);
-		p += s;
-		len -= s;
-	}
-	if (!dvb_fe_retrieve_stats(parms, DTV_SNR, &snr)) {
-		s = snprintf(p, len, "| snr %5d ", snr);
-		p += s;
-		len -= s;
-	}
-
-	if (!dvb_fe_retrieve_stats(parms, DTV_BER, &ber)) {
-		s = snprintf(p, len, "| ber %6d ", ber);
-		p += s;
-		len -= s;
-	}
-
-	if (!dvb_fe_retrieve_stats(parms, DTV_UNCORRECTED_BLOCKS,
-				    &ucb)) {
-		s = snprintf(p, len, "| ucb %6d ", ucb);
-		p += s;
-		len -= s;
-	}
-
-	if (!(status & FE_HAS_LOCK))
-		s = snprintf(p, len, "| failed\n");
-	else
-		s = snprintf(p, len, "\n");
-
-	fprintf(stderr, "%s", buf);
+	return 0;
 }
 
-static int check_frontend(struct dvb_v5_fe_parms *parms, int timeout)
+static int check_frontend(struct arguments *args, struct dvb_v5_fe_parms *parms)
 {
 	int rc, i;
 	fe_status_t status;
 
-	for (i = 0; i < timeout * 10; i++) {
+	args->n_status_lines = 0;
+	for (i = 0; i < args->timeout_multiply * 40; i++) {
 		rc = dvb_fe_get_stats(parms);
 		if (rc)
 			PERROR("dvb_fe_get_stats failed");
@@ -177,12 +177,15 @@ static int check_frontend(struct dvb_v5_fe_parms *parms, int timeout)
 		rc = dvb_fe_retrieve_stats(parms, DTV_STATUS, &status);
 		if (rc)
 			status = 0;
+		print_frontend_stats(args, parms);
 		if (status & FE_HAS_LOCK)
 			break;
 		usleep(100000);
 	};
 
-	print_frontend_stats(parms);
+	if (isatty(STDERR_FILENO)) {
+		fprintf(stderr, "\x1b[37m");
+	}
 
 	return (status & FE_HAS_LOCK) ? 0 : -1;
 }
@@ -466,7 +469,7 @@ static int run_scan(struct arguments *args,
 		if (verbose)
 			dvb_fe_prt_parms(parms);
 
-		rc = check_frontend(parms, 4);
+		rc = check_frontend(args, parms);
 		if (rc < 0)
 			continue;
 
@@ -589,8 +592,11 @@ int main(int argc, char **argv)
 	args.output = DEFAULT_OUTPUT;
 	args.input_format = FILE_DVBV5;
 	args.output_format = FILE_DVBV5;
+	args.timeout_multiply = 1;
 
 	argp_parse(&argp, argc, argv, 0, &idx, &args);
+	if (args.timeout_multiply == 0)
+		args.timeout_multiply = 1;
 
 	if (args.lnb_name) {
 		lnb = dvb_sat_search_lnb(args.lnb_name);
