@@ -32,6 +32,7 @@
 #include <errno.h>
 #include <signal.h>
 #include <argp.h>
+#include <sys/time.h>
 
 #include <config.h>
 
@@ -51,33 +52,43 @@ struct arguments {
 	char *filename;
 	unsigned adapter, frontend, demux, get_detected, get_nit;
 	int force_dvbv3, lnb, sat_number;
-	unsigned diseqc_wait, silent, frontend_only, freq_bpf;
+	unsigned diseqc_wait, silent, verbose, frontend_only, freq_bpf;
 	unsigned timeout, dvr, rec_psi, exit_after_tuning;
 	unsigned record;
-	unsigned n_apid, n_vpid;
+	unsigned n_apid, n_vpid, all_pids;
 	enum file_formats input_format, output_format;
+	unsigned traffic_monitor, low_traffic;
+	char *search;
+
+	/* Used by status print */
+	unsigned n_status_lines;
 };
 
 static const struct argp_option options[] = {
+	{"dvbv3",	'3', NULL,			0, "Use DVBv3 only", 0},
 	{"adapter",	'a', "adapter#",		0, "use given adapter (default 0)", 0},
-	{"frontend",	'f', "frontend#",		0, "use given frontend (default 0)", 0},
-	{"demux",	'd', "demux#",			0, "use given demux (default 0)", 0},
 	{"audio_pid",	'A', "audio_pid#",		0, "audio pid program to use (default 0)", 0},
-	{"video_pid",	'V', "video_pid#",		0, "video pid program to use (default 0)", 0},
-	{"lnbf",	'l', "LNBf_type",		0, "type of LNBf to use. 'help' lists the available ones", 0},
-	{"sat_number",	'S', "satellite_number",	0, "satellite number. If not specified, disable DISEqC", 0},
-	{"freq_bpf",	'U', "frequency",		0, "SCR/Unicable band-pass filter frequency to use, in kHz", 0},
-	{"wait",	'W', "time",			0, "adds additional wait time for DISEqC command completion", 0},
 	{"channels",	'c', "file",			0, "read channels list from 'file'", 0},
-	{"exit",	'x', NULL,			0, "exit after tuning", 0},
-	{"record",	'r', NULL,			0, "set up /dev/dvb/adapterX/dvr0 for TS recording", 0},
-	{"pat",		'p', NULL,			0, "add pat and pmt to TS recording (implies -r)", 0},
-	{"silence",	's', NULL,			0, "increases silence (can be used more than once)", 0},
+	{"demux",	'd', "demux#",			0, "use given demux (default 0)", 0},
+	{"frontend",	'f', "frontend#",		0, "use given frontend (default 0)", 0},
 	{"frontend",	'F', NULL,			0, "set up frontend only, don't touch demux", 0},
-	{"timeout",	't', "seconds",			0, "timeout for zapping and for recording", 0},
-	{"output",	'o', "file",			0, "output filename (use -o - for stdout)", 0},
 	{"input-format", 'I',	"format",		0, "Input format: ZAP, CHANNEL, DVBV5 (default: DVBV5)", 0},
-	{"dvbv3",	'3',	0,			0, "Use DVBv3 only", 0},
+	{"lnbf",	'l', "LNBf_type",		0, "type of LNBf to use. 'help' lists the available ones", 0},
+	{"search",	'L', NULL,			0, "search/look for a string inside the traffic", 0},
+	{"monitor",	'm', NULL,			0, "monitors de DVB traffic", 0},
+	{"output",	'o', "file",			0, "output filename (use -o - for stdout)", 0},
+	{"pat",		'p', NULL,			0, "add pat and pmt to TS recording (implies -r)", 0},
+	{"all-pids",	'P', NULL,			0, "don't filter any pids. Instead, outputs all of them", 0 },
+	{"record",	'r', NULL,			0, "set up /dev/dvb/adapterX/dvr0 for TS recording", 0},
+	{"silence",	's', NULL,			0, "increases silence (can be used more than once)", 0},
+	{"sat_number",	'S', "satellite_number",	0, "satellite number. If not specified, disable DISEqC", 0},
+	{"timeout",	't', "seconds",			0, "timeout for zapping and for recording", 0},
+	{"freq_bpf",	'U', "frequency",		0, "SCR/Unicable band-pass filter frequency to use, in kHz", 0},
+	{"verbose",	'v', NULL,			0, "verbose debug messages (can be used more than once)", 0},
+	{"video_pid",	'V', "video_pid#",		0, "video pid program to use (default 0)", 0},
+	{"wait",	'W', "time",			0, "adds additional wait time for DISEqC command completion", 0},
+	{"exit",	'x', NULL,			0, "exit after tuning", 0},
+	{"low_traffic",	'X', NULL,			0, "also shows DVB traffic with less then 1 packet per second", 0},
 	{ 0, 0, 0, 0, 0, 0 }
 };
 
@@ -154,6 +165,15 @@ static int parse(struct arguments *args,
 		return -3;
 	}
 
+	if (entry->lnb) {
+		int lnb = dvb_sat_search_lnb(entry->lnb);
+		if (lnb == -1) {
+			PERROR("unknown LNB %s\n", entry->lnb);
+			return -1;
+		}
+		parms->lnb = dvb_sat_get_lnb(lnb);
+	}
+
 	if (entry->video_pid) {
 		if (args->n_vpid < entry->video_pid_len)
 			*vpid = entry->video_pid[args->n_vpid];
@@ -272,23 +292,19 @@ static void do_timeout(int x)
 	}
 }
 
-static char *sig_bits[7] = {
-	[0] = "RF sig",
-	[1] = "Carrier",
-	[2] = "Viterbi",
-	[3] = "Sync",
-	[4] = "Lock",
-	[5] = "Timeout",
-	[6] = "Reinit",
-};
-
-static int print_frontend_stats(struct dvb_v5_fe_parms *parms)
+static int print_frontend_stats(FILE *fd,
+				struct arguments *args,
+				struct dvb_v5_fe_parms *parms)
 {
-	char buf[256], *p = buf;
-	int rc, i, s, len = sizeof(buf);
-	fe_status_t status;
-	uint32_t snr = 0, _signal = 0;
-	uint32_t ber = 0, ucb = 0;
+	char buf[512], *p;
+	int rc, i, len, show;
+	uint32_t status = 0;
+
+	/* Move cursor up and cleans down */
+	if (isatty(fileno(fd)) && args->n_status_lines)
+		fprintf(fd, "\r\x1b[%dA\x1b[J", args->n_status_lines);
+
+	args->n_status_lines = 0;
 
 	rc = dvb_fe_get_stats(parms);
 	if (rc) {
@@ -296,59 +312,53 @@ static int print_frontend_stats(struct dvb_v5_fe_parms *parms)
 		return -1;
 	}
 
-	if (dvb_fe_retrieve_stats(parms, DTV_STATUS, &status)) {
-		fprintf(stderr, "Error: dvb_fe_get_stats failed: no status\n");
-		return -1;
-	}
+	p = buf;
+	len = sizeof(buf);
+	dvb_fe_snprintf_stat(parms,  DTV_STATUS, NULL, 0, &p, &len, &show);
 
-	/* Get the most significative status */
-	for (i = ARRAY_SIZE(sig_bits) - 1; i >= 0 ; i--) {
-		if ((1 << i) & status) {
-			s = snprintf(p, len, "%-7s ", sig_bits[i]);
-			break;
+	for (i = 0; i < MAX_DTV_STATS; i++) {
+		show = 1;
+
+		dvb_fe_snprintf_stat(parms, DTV_QUALITY, "Quality",
+				     i, &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_STAT_SIGNAL_STRENGTH, "Signal",
+				     i, &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_STAT_CNR, "C/N",
+				     i, &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_STAT_ERROR_BLOCK_COUNT, "UCB",
+				     i,  &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_BER, "postBER",
+				     i,  &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_PRE_BER, "preBER",
+				     i,  &p, &len, &show);
+
+		dvb_fe_snprintf_stat(parms, DTV_PER, "PER",
+				     i,  &p, &len, &show);
+
+		if (p != buf) {
+			if (args->n_status_lines)
+				fprintf(fd, "\t%s\n", buf);
+			else
+				fprintf(fd, "%s\n", buf);
+
+			args->n_status_lines++;
+
+			p = buf;
+			len = sizeof(buf);
 		}
 	}
-	if (i < 0)
-		s = snprintf(p, len, "%7s ", "");
-	p += s;
-	len -= s;
 
-	/* Add the status bits */
-	s = snprintf(p, len, "(0x%02x) ", status);
-	p += s;
-	len -= s;
-
-	if (!dvb_fe_retrieve_stats(parms, DTV_SIGNAL_STRENGTH, &_signal)) {
-		s = snprintf(p, len, "| signal %03.2f%% ",
-			     (_signal * 100.0) / 0xffff);
-		p += s;
-		len -= s;
-	}
-	if (!dvb_fe_retrieve_stats(parms, DTV_SNR, &snr)) {
-		s = snprintf(p, len, "| snr %5d ", snr);
-		p += s;
-		len -= s;
-	}
-
-	if (!dvb_fe_retrieve_stats(parms, DTV_BER, &ber)) {
-		s = snprintf(p, len, "| ber %6d ", ber);
-		p += s;
-		len -= s;
-	}
-
-	if (!dvb_fe_retrieve_stats(parms, DTV_UNCORRECTED_BLOCKS,
-				    &ucb)) {
-		s = snprintf(p, len, "| ucb %6d ", ucb);
-		p += s;
-		len -= s;
-	}
-
-	fprintf(stderr, "\r%s", buf);
-	fflush(stderr);
+	fflush(fd);
 
 	/* While not lock, display status on a new line */
-	if (!(status & FE_HAS_LOCK))
-		fprintf(stderr, "\n");
+	dvb_fe_retrieve_stats(parms, DTV_STATUS, &status);
+	if (!isatty(fileno(fd)) || !(status & FE_HAS_LOCK))
+		fprintf(fd, "\n");
 
 	return 0;
 }
@@ -365,13 +375,13 @@ static int check_frontend(struct arguments *args,
 
 		rc = dvb_fe_retrieve_stats(parms, DTV_STATUS, &status);
 		if (!args->silent)
-			print_frontend_stats(parms);
+			print_frontend_stats(stderr, args, parms);
 		if (args->exit_after_tuning && (status & FE_HAS_LOCK))
 			break;
 		usleep(1000000);
 	} while (!timeout_flag);
 	if (args->silent < 2)
-		print_frontend_stats(parms);
+		print_frontend_stats(stderr, args, parms);
 
 	return 0;
 }
@@ -455,6 +465,9 @@ static error_t parse_opt(int k, char *optarg, struct argp_state *state)
 	case 's':
 		args->silent++;
 		break;
+	case 'v':
+		args->verbose++;
+		break;
 	case 'F':
 		args->frontend_only = 1;
 		break;
@@ -463,15 +476,161 @@ static error_t parse_opt(int k, char *optarg, struct argp_state *state)
 		break;
 	case 'V':
 		args->n_vpid = strtoul(optarg, NULL, 0);
+		break;
+	case 'P':
+		args->all_pids++;
+		break;
 	case '3':
 		args->force_dvbv3 = 1;
 		break;
+	case 'm':
+		args->traffic_monitor = 1;
+		break;
+	case 'X':
+		args->low_traffic = 1;
+		break;
+	case 'L':
+		args->search = strdup(optarg);
+		break;
 	default:
 		return ARGP_ERR_UNKNOWN;
 	};
 	return 0;
 }
 
+#define BSIZE 188
+
+int do_traffic_monitor(struct arguments *args,
+		    struct dvb_v5_fe_parms *parms)
+{
+	int fd, dvr_fd;
+	long long unsigned pidt[0x2001], wait;
+	int packets = 0;
+	struct timeval startt;
+
+	memset(pidt, 0, sizeof(pidt));
+
+	args->exit_after_tuning = 1;
+	check_frontend(args, parms);
+
+	if ((dvr_fd = open(args->dvr_dev, O_RDONLY)) < 0) {
+		PERROR("failed opening '%s'", args->dvr_dev);
+		return -1;
+	}
+
+	if (ioctl(dvr_fd, DMX_SET_BUFFER_SIZE, 1024 * 1024) == -1)
+		perror("DMX_SET_BUFFER_SIZE failed");
+
+	if ((fd = open(args->demux_dev, O_RDWR)) < 0) {
+		PERROR("failed opening '%s'", args->demux_dev);
+		return -1;
+	}
+
+	if (args->silent < 2)
+		fprintf(stderr, "  dvb_set_pesfilter to 0x2000\n");
+	if (dvb_set_pesfilter(fd, 0x2000, DMX_PES_OTHER,
+			      DMX_OUT_TS_TAP, 0) < 0) {
+		PERROR("couldn't set filter");
+		return -1;
+	}
+
+	gettimeofday(&startt, 0);
+	wait = 1000;
+
+	while (1) {
+		unsigned char buffer[BSIZE];
+		int pid, ok;
+		ssize_t r;
+		if ((r = read(dvr_fd, buffer, BSIZE)) <= 0) {
+			if (errno == EOVERFLOW) {
+				struct timeval now;
+				int diff;
+				gettimeofday(&now, 0);
+				diff =
+				    (now.tv_sec - startt.tv_sec) * 1000 +
+				    (now.tv_usec - startt.tv_usec) / 1000;
+				fprintf(stderr, "%.2fs: buffer overrun\n", diff / 1000.);
+				continue;
+			}
+			perror("read");
+			break;
+		}
+		if (r != BSIZE) {
+			fprintf(stderr, "dvbtraffic: only read %zd bytes\n", r);
+			break;
+		}
+		if (buffer[0] != 0x47) {
+			continue;
+			printf("desync (%x)\n", buffer[0]);
+			while (buffer[0] != 0x47)
+				read(fd, buffer, 1);
+			continue;
+		}
+		ok = 1;
+		pid = ((((unsigned) buffer[1]) << 8) |
+		       ((unsigned) buffer[2])) & 0x1FFF;
+
+		if (args->search) {
+			int i, sl = strlen(args->search);
+			ok = 0;
+			if (pid != 0x1fff) {
+				for (i = 0; i < (188 - sl); ++i) {
+					if (!memcmp(buffer + i, args->search, sl))
+						ok = 1;
+				}
+			}
+		}
+
+		if (ok) {
+			pidt[pid]++;
+			pidt[0x2000]++;
+		}
+
+		packets++;
+
+		if (!(packets & 0xFF)) {
+			struct timeval now;
+			int diff;
+			gettimeofday(&now, 0);
+			diff =
+			    (now.tv_sec - startt.tv_sec) * 1000 +
+			    (now.tv_usec - startt.tv_usec) / 1000;
+			if (diff > wait) {
+				if (isatty(STDOUT_FILENO))
+			                printf("\x1b[1H\x1b[2J");
+
+				args->n_status_lines = 0;
+				printf(" PID          FREQ         SPEED       TOTAL\n");
+				int _pid = 0;
+				for (_pid = 0; _pid < 0x2000; _pid++) {
+					if (pidt[_pid]) {
+						if (!args->low_traffic && (pidt[_pid] * 1000. / diff) < 1)
+							continue;
+						printf("%04x %9.2f p/s %8.1f Kbps ",
+						     _pid,
+						     pidt[_pid] * 1000. / diff,
+						     pidt[_pid] * 1000. / diff * 8 * 188 / 1024);
+						if (pidt[_pid] * 188 / 1024)
+							printf("%8llu KB\n", pidt[_pid] * 188 / 1024);
+						else
+							printf(" %8llu B\n", pidt[_pid] * 188);
+					}
+				}
+				/* 0x2000 is the total traffic */
+				printf("TOT %10.2f p/s %8.1f Kbps %8llu KB\n",
+				     pidt[_pid] * 1000. / diff,
+				     pidt[_pid] * 1000. / diff * 8 * 188 / 1024,
+				     pidt[_pid] * 188 / 1024);
+				printf("\n\n");
+				print_frontend_stats(stdout, args, parms);
+				wait += 1000;
+			}
+		}
+	}
+	close (fd);
+	return 0;
+}
+
 int main(int argc, char **argv)
 {
 	struct arguments args;
@@ -542,7 +701,7 @@ int main(int argc, char **argv)
 	}
 	fprintf(stderr, "reading channels from file '%s'\n", args.confname);
 
-	parms = dvb_fe_open(args.adapter, args.frontend, 0, args.force_dvbv3);
+	parms = dvb_fe_open(args.adapter, args.frontend, args.verbose, args.force_dvbv3);
 	if (!parms)
 		return -1;
 	if (lnb)
@@ -564,6 +723,9 @@ int main(int argc, char **argv)
 		return 0;
 	}
 
+	if (args.traffic_monitor)
+		return do_traffic_monitor(&args, parms);
+
 	if (args.rec_psi) {
 		if (sid < 0) {
 			fprintf(stderr, "Service id 0x%04x was not specified at the file\n",
@@ -596,9 +758,17 @@ int main(int argc, char **argv)
 			return -1;
 	}
 
+	if (args.all_pids++) {
+		vpid = 0x2000;
+		apid = 0;
+	}
 	if (vpid >= 0) {
-		if (args.silent < 2)
-			fprintf(stderr, "video pid %d\n", vpid);
+		if (args.silent < 2) {
+			if (vpid == 0x2000)
+				fprintf(stderr, "pass all PID's to TS\n");
+			else
+				fprintf(stderr, "video pid %d\n", vpid);
+		}
 		if ((video_fd = open(args.demux_dev, O_RDWR)) < 0) {
 			PERROR("failed opening '%s'", args.demux_dev);
 			return -1;
@@ -658,12 +828,12 @@ int main(int argc, char **argv)
 			return -1;
 		}
 		if (args.silent < 2)
-			print_frontend_stats(parms);
+			print_frontend_stats(stderr, &args, parms);
 
 		copy_to_file(dvr_fd, file_fd, args.timeout, args.silent);
 
 		if (args.silent < 2)
-			print_frontend_stats(parms);
+			print_frontend_stats(stderr, &args, parms);
 	} else {
 		check_frontend(&args, parms);
 	}
