@@ -91,9 +91,9 @@ enum ir_protocols {
 
 static int parse_code(char *string)
 {
-	struct parse_key *p;
+	struct parse_event *p;
 
-	for (p = keynames; p->name != NULL; p++) {
+	for (p = key_events; p->name != NULL; p++) {
 		if (!strcasecmp(p->name, string))
 			return p->value;
 	}
@@ -207,13 +207,19 @@ static error_t parse_keyfile(char *fname, char **table)
 			p++;
 			p = strtok(p, "\n\t =:");
 			do {
+				if (!p)
+					goto err_einval;
 				if (!strcmp(p, "table")) {
 					p = strtok(NULL,"\n, ");
+					if (!p)
+						goto err_einval;
 					*table = malloc(strlen(p) + 1);
 					strcpy(*table, p);
 				} else if (!strcmp(p, "type")) {
 					p = strtok(NULL, " ,\n");
 					do {
+						if (!p)
+							goto err_einval;
 						if (!strcasecmp(p,"rc5") || !strcasecmp(p,"rc-5"))
 							ch_proto |= RC_5;
 						else if (!strcasecmp(p,"rc6") || !strcasecmp(p,"rc-6"))
@@ -431,7 +437,7 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 			nextkey->codes[1] = key;
 
 			if (debug)
-				fprintf(stderr, "scancode %u=%u\n",
+				fprintf(stderr, "scancode 0x%04x=%u\n",
 					nextkey->codes[0], nextkey->codes[1]);
 
 			nextkey->next = calloc(1, sizeof(keys));
@@ -447,6 +453,8 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 	case 'p':
 		p = strtok(arg, ",;");
 		do {
+			if (!p)
+				goto err_inval;
 			if (!strcasecmp(p,"rc5") || !strcasecmp(p,"rc-5"))
 				ch_proto |= RC_5;
 			else if (!strcasecmp(p,"rc6") || !strcasecmp(p,"rc-6"))
@@ -488,9 +496,9 @@ static struct argp argp = {
 
 static void prtcode(int *codes)
 {
-	struct parse_key *p;
+	struct parse_event *p;
 
-	for (p = keynames; p->name != NULL; p++) {
+	for (p = key_events; p->name != NULL; p++) {
 		if (p->value == (unsigned)codes[1]) {
 			printf("scancode 0x%04x = %s (0x%02x)\n", codes[0], p->name, codes[1]);
 			return;
@@ -498,9 +506,9 @@ static void prtcode(int *codes)
 	}
 
 	if (isprint (codes[1]))
-		printf("scancode %d = '%c' (0x%02x)\n", codes[0], codes[1], codes[1]);
+		printf("scancode 0x%04x = '%c' (0x%02x)\n", codes[0], codes[1], codes[1]);
 	else
-		printf("scancode %d = 0x%02x\n", codes[0], codes[1]);
+		printf("scancode 0x%04x = 0x%02x\n", codes[0], codes[1]);
 }
 
 static void free_names(struct sysfs_names *names)
@@ -813,14 +821,19 @@ static int v1_get_sw_enabled_protocol(char *dirname)
 		return 0;
 	}
 
-	p = strtok(buf, " \n");
-	rc = atoi(p);
-
 	if (fclose(fp)) {
 		perror(name);
 		return errno;
 	}
 
+	p = strtok(buf, " \n");
+	if (!p) {
+		fprintf(stderr, "%s has invalid content: '%s'\n", name, buf);
+		return 0;
+	}
+
+	rc = atoi(p);
+
 	if (debug)
 		fprintf(stderr, "protocol %s is %s\n",
 			name, rc? "enabled" : "disabled");
@@ -1236,6 +1249,18 @@ static void display_proto(struct rc_device *rc_dev)
 	fprintf(stderr, "\n");
 }
 
+
+static char *get_event_name(struct parse_event *event, u_int16_t code)
+{
+	struct parse_event *p;
+
+	for (p = event; p->name != NULL; p++) {
+		if (p->value == code)
+			return p->name;
+	}
+	return "";
+}
+
 static void test_event(int fd)
 {
 	struct input_event ev[64];
@@ -1251,45 +1276,52 @@ static void test_event(int fd)
 		}
 
 		for (i = 0; i < rd / sizeof(struct input_event); i++) {
+			printf("%ld.%06ld: event type %s(0x%02x)",
+				ev[i].time.tv_sec, ev[i].time.tv_usec,
+				get_event_name(events_type, ev[i].type), ev[i].type);
+
 			switch (ev[i].type) {
-			case EV_MSC:
-				if (ev[i].code != MSC_SCAN)
-					break;
-				printf("%ld.%06ld: event MSC: scancode = %02x\n",
-					ev[i].time.tv_sec, ev[i].time.tv_usec, ev[i].value);
+			case EV_SYN:
+				printf(".\n");
 				break;
-			case EV_KEY: 			{
-				struct parse_key *p;
-				char *name = "";
-
-				printf("%ld.%06ld: event key %s: ",
-					ev[i].time.tv_sec, ev[i].time.tv_usec,
-					(ev[i].value == 0) ? "up" : "down"
-					);
-
-				for (p = keynames; p->name != NULL; p++) {
-					if (p->value == ev[i].code) {
-						name = p->name;
-						break;
-					}
-				}
-				printf("%s (0x%04x)\n", name, ev[i].code);
-
+			case EV_KEY:
+				printf(" key_%s: %s(0x%04x)\n",
+					(ev[i].value == 0) ? "up" : "down",
+					get_event_name(key_events, ev[i].code),
+					ev[i].type);
 				break;
-			}
-			case EV_REP:
-				printf("%ld.%06ld: event repeat: %d\n",
-					ev[i].time.tv_sec, ev[i].time.tv_usec,
+			case EV_REL:
+				printf(": %s (0x%04x) value=%d\n",
+					get_event_name(rel_events, ev[i].code),
+					ev[i].type,
 					ev[i].value);
 				break;
-			case EV_SYN:
-				printf("%ld.%06ld: event sync\n",
-					ev[i].time.tv_sec, ev[i].time.tv_usec);
+			case EV_ABS:
+				printf(": %s (0x%04x) value=%d\n",
+					get_event_name(abs_events, ev[i].code),
+					ev[i].type,
+					ev[i].value);
 				break;
+			case EV_MSC:
+				if (ev[i].code == MSC_SCAN)
+					printf(": scancode = 0x%02x\n", ev[i].value);
+				else
+					printf(": code = %s(0x%02x), value = %d\n",
+						get_event_name(msc_events, ev[i].code),
+						ev[i].code, ev[i].value);
+				break;
+			case EV_REP:
+				printf(": value = %d\n", ev[i].value);
+				break;
+			case EV_SW:
+			case EV_LED:
+			case EV_SND:
+			case EV_FF:
+			case EV_PWR:
+			case EV_FF_STATUS:
 			default:
-				printf("%ld.%06ld: event type %d: value: %d\n",
-					ev[i].time.tv_sec, ev[i].time.tv_usec,
-					ev[i].type, ev[i].value);
+				printf(": code = 0x%02x, value = %d\n",
+					ev[i].code, ev[i].value);
 				break;
 			}
 		}
@@ -1331,7 +1363,7 @@ static void display_table_v2(struct rc_device *rc_dev, int fd)
 			break;
 
 		/* FIXME: Extend it to support scancodes > 32 bits */
-		codes[0] = ((u_int32_t *)entry.scancode)[0];
+		memcpy(&codes[0], entry.scancode, sizeof(codes[0]));
 		codes[1] = entry.keycode;
 
 		prtcode(codes);
