@@ -42,8 +42,6 @@ GeneralTab::GeneralTab(const QString &device, v4l2 &fd, int n, QWidget *parent)
 	m_audioInput(NULL),
 	m_tvStandard(NULL),
 	m_qryStandard(NULL),
-	m_videoPreset(NULL),
-	m_qryPreset(NULL),
 	m_videoTimings(NULL),
 	m_qryTimings(NULL),
 	m_freq(NULL),
@@ -75,7 +73,6 @@ GeneralTab::GeneralTab(const QString &device, v4l2 &fd, int n, QWidget *parent)
 
 	v4l2_input vin;
 	bool needsStd = false;
-	bool needsPreset = false;
 	bool needsTimings = false;
 
 	if (m_tuner.capability && m_tuner.capability & V4L2_TUNER_CAP_LOW)
@@ -92,8 +89,6 @@ GeneralTab::GeneralTab(const QString &device, v4l2 &fd, int n, QWidget *parent)
 			m_videoInput->addItem((char *)vin.name);
 			if (vin.capabilities & V4L2_IN_CAP_STD)
 				needsStd = true;
-			if (vin.capabilities & V4L2_IN_CAP_PRESETS)
-				needsPreset = true;
 			if (vin.capabilities & V4L2_IN_CAP_DV_TIMINGS)
 				needsTimings = true;
 		} while (enum_input(vin));
@@ -138,27 +133,19 @@ GeneralTab::GeneralTab(const QString &device, v4l2 &fd, int n, QWidget *parent)
 	}
 
 	if (needsStd) {
+		v4l2_std_id tmp;
+
 		addLabel("TV Standard");
 		m_tvStandard = new QComboBox(parent);
 		addWidget(m_tvStandard);
 		connect(m_tvStandard, SIGNAL(activated(int)), SLOT(standardChanged(int)));
 		refreshStandards();
-		addLabel("");
-		m_qryStandard = new QPushButton("Query Standard", parent);
-		addWidget(m_qryStandard);
-		connect(m_qryStandard, SIGNAL(clicked()), SLOT(qryStdClicked()));
-	}
-
-	if (needsPreset) {
-		addLabel("Video Preset");
-		m_videoPreset = new QComboBox(parent);
-		addWidget(m_videoPreset);
-		connect(m_videoPreset, SIGNAL(activated(int)), SLOT(presetChanged(int)));
-		refreshPresets();
-		addLabel("");
-		m_qryPreset = new QPushButton("Query Preset", parent);
-		addWidget(m_qryPreset);
-		connect(m_qryPreset, SIGNAL(clicked()), SLOT(qryPresetClicked()));
+		if (ioctl_exists(VIDIOC_QUERYSTD, &tmp)) {
+			addLabel("");
+			m_qryStandard = new QPushButton("Query Standard", parent);
+			addWidget(m_qryStandard);
+			connect(m_qryStandard, SIGNAL(clicked()), SLOT(qryStdClicked()));
+		}
 	}
 
 	if (needsTimings) {
@@ -440,15 +427,6 @@ void GeneralTab::standardChanged(int std)
 	updateStandard();
 }
 
-void GeneralTab::presetChanged(int index)
-{
-	v4l2_dv_enum_preset preset;
-
-	enum_dv_preset(preset, true, index);
-	s_dv_preset(preset.preset);
-	updatePreset();
-}
-
 void GeneralTab::timingsChanged(int index)
 {
 	v4l2_enum_dv_timings timings;
@@ -476,7 +454,7 @@ void GeneralTab::freqChanged()
 {
 	double f = m_freq->text().toDouble();
 
-	s_frequency(f * 16, m_tuner.capability & V4L2_TUNER_CAP_LOW);
+	s_frequency(f * 16, m_isRadio);
 }
 
 void GeneralTab::audioModeChanged(int)
@@ -633,13 +611,8 @@ void GeneralTab::updateVideoInput()
 		refreshStandards();
 		updateStandard();
 		m_tvStandard->setEnabled(in.capabilities & V4L2_IN_CAP_STD);
-		m_qryStandard->setEnabled(in.capabilities & V4L2_IN_CAP_STD);
-	}
-	if (m_videoPreset) {
-		refreshPresets();
-		updatePreset();
-		m_videoPreset->setEnabled(in.capabilities & V4L2_IN_CAP_PRESETS);
-		m_qryPreset->setEnabled(in.capabilities & V4L2_IN_CAP_PRESETS);
+		if (m_qryStandard)
+			m_qryStandard->setEnabled(in.capabilities & V4L2_IN_CAP_STD);
 	}
 	if (m_videoTimings) {
 		refreshTimings();
@@ -660,11 +633,8 @@ void GeneralTab::updateVideoOutput()
 	m_videoOutput->setCurrentIndex(output);
 	if (m_tvStandard) {
 		m_tvStandard->setEnabled(out.capabilities & V4L2_OUT_CAP_STD);
-		m_qryStandard->setEnabled(out.capabilities & V4L2_OUT_CAP_STD);
-	}
-	if (m_videoPreset) {
-		m_videoPreset->setEnabled(out.capabilities & V4L2_OUT_CAP_PRESETS);
-		m_qryPreset->setEnabled(out.capabilities & V4L2_OUT_CAP_PRESETS);
+		if (m_qryStandard)
+			m_qryStandard->setEnabled(out.capabilities & V4L2_OUT_CAP_STD);
 	}
 	if (m_videoTimings) {
 		m_videoTimings->setEnabled(out.capabilities & V4L2_OUT_CAP_DV_TIMINGS);
@@ -747,56 +717,17 @@ void GeneralTab::qryStdClicked()
 {
 	v4l2_std_id std;
 
-	if (query_std(std)) {
+	if (!query_std(std))
+		return;
+
+	if (std == V4L2_STD_ALL) {
+		info("No standard detected\n");
+	} else {
 		s_std(std);
 		updateStandard();
 	}
 }
 
-void GeneralTab::refreshPresets()
-{
-	v4l2_dv_enum_preset preset;
-	m_videoPreset->clear();
-	if (enum_dv_preset(preset, true)) {
-		do {
-			m_videoPreset->addItem((char *)preset.name);
-		} while (enum_dv_preset(preset));
-	}
-}
-
-void GeneralTab::updatePreset()
-{
-	__u32 preset;
-	v4l2_dv_enum_preset p;
-	QString what;
-
-	g_dv_preset(preset);
-	if (enum_dv_preset(p, true)) {
-		do {
-			if (p.preset == preset)
-				break;
-		} while (enum_dv_preset(p));
-	}
-	if (p.preset != preset)
-		return;
-	m_videoPreset->setCurrentIndex(p.index);
-	what.sprintf("Video Preset (%u)\n"
-		"Frame %ux%u\n",
-		p.preset, p.width, p.height);
-	m_videoPreset->setWhatsThis(what);
-	updateVidCapFormat();
-}
-
-void GeneralTab::qryPresetClicked()
-{
-	v4l2_dv_preset preset;
-
-	if (query_dv_preset(preset)) {
-		s_dv_preset(preset.preset);
-		updatePreset();
-	}
-}
-
 void GeneralTab::refreshTimings()
 {
 	v4l2_enum_dv_timings timings;
@@ -804,14 +735,19 @@ void GeneralTab::refreshTimings()
 	if (enum_dv_timings(timings, true)) {
 		do {
 			v4l2_bt_timings &bt = timings.timings.bt;
+			double tot_height = bt.height +
+				bt.vfrontporch + bt.vsync + bt.vbackporch +
+				bt.il_vfrontporch + bt.il_vsync + bt.il_vbackporch;
+			double tot_width = bt.width +
+				bt.hfrontporch + bt.hsync + bt.hbackporch;
 			char buf[100];
 
-			sprintf(buf, "%dx%d%c%.2f", bt.width, bt.height,
-					bt.interlaced ? 'i' : 'p',
-					(double)bt.pixelclock /
-						((bt.width + bt.hfrontporch + bt.hsync + bt.hbackporch) *
-						 (bt.height + bt.vfrontporch + bt.vsync + bt.vbackporch +
-						  bt.il_vfrontporch + bt.il_vsync + bt.il_vbackporch)));
+			if (bt.interlaced)
+				sprintf(buf, "%dx%di%.2f", bt.width, bt.height,
+					(double)bt.pixelclock / (tot_width * (tot_height / 2)));
+			else
+				sprintf(buf, "%dx%dp%.2f", bt.width, bt.height,
+					(double)bt.pixelclock / (tot_width * tot_height));
 			m_videoTimings->addItem(buf);
 		} while (enum_dv_timings(timings));
 	}
@@ -913,6 +849,12 @@ void GeneralTab::updateFrameSize()
 
 		m_frameWidth->setEnabled(false);
 		m_frameHeight->setEnabled(false);
+		m_frameWidth->setMinimum(m_width);
+		m_frameWidth->setMaximum(m_width);
+		m_frameWidth->setValue(m_width);
+		m_frameHeight->setMinimum(m_height);
+		m_frameHeight->setMaximum(m_height);
+		m_frameHeight->setValue(m_height);
 		m_frameSize->setEnabled(true);
 		updateFrameInterval();
 		return;
