@@ -86,7 +86,7 @@ static std::string fract2sec(const struct v4l2_fract &f)
 {
 	char buf[100];
 
-	sprintf(buf, "%.3f s", (1.0 * f.numerator) / f.denominator);
+	sprintf(buf, "%.3f", (1.0 * f.numerator) / f.denominator);
 	return buf;
 }
 
@@ -94,7 +94,7 @@ static std::string fract2fps(const struct v4l2_fract &f)
 {
 	char buf[100];
 
-	sprintf(buf, "%.3f fps", (1.0 * f.denominator) / f.numerator);
+	sprintf(buf, "%.3f", (1.0 * f.denominator) / f.numerator);
 	return buf;
 }
 
@@ -119,18 +119,15 @@ static void print_frmival(const struct v4l2_frmivalenum &frmival, const char *pr
 {
 	printf("%s\tInterval: %s ", prefix, frmtype2s(frmival.type).c_str());
 	if (frmival.type == V4L2_FRMIVAL_TYPE_DISCRETE) {
-		printf("%s (%s)\n", fract2sec(frmival.discrete).c_str(),
+		printf("%ss (%s fps)\n", fract2sec(frmival.discrete).c_str(),
 				fract2fps(frmival.discrete).c_str());
 	} else if (frmival.type == V4L2_FRMIVAL_TYPE_STEPWISE) {
-		printf("%s - %s with step %s\n",
+		printf("%ss - %ss with step %ss (%s-%s fps)\n",
 				fract2sec(frmival.stepwise.min).c_str(),
 				fract2sec(frmival.stepwise.max).c_str(),
-				fract2sec(frmival.stepwise.step).c_str());
-		printf("%s\t            : ", prefix);
-		printf("(%s - %s with step %s)\n",
-				fract2fps(frmival.stepwise.min).c_str(),
+				fract2sec(frmival.stepwise.step).c_str(),
 				fract2fps(frmival.stepwise.max).c_str(),
-				fract2fps(frmival.stepwise.step).c_str());
+				fract2fps(frmival.stepwise.min).c_str());
 	}
 }
 
@@ -259,7 +256,7 @@ void vidcap_set(int fd)
 				if (in_vfmt.fmt.pix.pixelformat < 256) {
 					in_vfmt.fmt.pix.pixelformat =
 						find_pixel_format(fd, in_vfmt.fmt.pix.pixelformat,
-								  false);
+								  false, false);
 				}
 			}
 			/* G_FMT might return a bytesperline value > width,
@@ -289,7 +286,7 @@ void vidcap_set(int fd)
 				if (in_vfmt.fmt.pix_mp.pixelformat < 256) {
 					in_vfmt.fmt.pix_mp.pixelformat =
 						find_pixel_format(fd, in_vfmt.fmt.pix_mp.pixelformat,
-								  true);
+								  false, true);
 				}
 			}
 			/* G_FMT might return bytesperline values > width,
