@@ -99,7 +99,7 @@ void vidout_set(int fd)
 				if (in_vfmt.fmt.pix.pixelformat < 256) {
 					in_vfmt.fmt.pix.pixelformat =
 						find_pixel_format(fd, in_vfmt.fmt.pix.pixelformat,
-								  false);
+								  true, false);
 				}
 			}
 			/* G_FMT might return a bytesperline value > width,
@@ -130,7 +130,7 @@ void vidout_set(int fd)
 				if (in_vfmt.fmt.pix_mp.pixelformat < 256) {
 					in_vfmt.fmt.pix_mp.pixelformat =
 						find_pixel_format(fd, in_vfmt.fmt.pix_mp.pixelformat,
-								  true);
+								  true, true);
 				}
 			}
 			/* G_FMT might return bytesperline values > width,
